<?php
/**
*
* @package hcode: code sintax highlighter
* @copyright (c) 2014 c61 c61@yandex.ru http://c61.su
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*/
namespace c61\hcode\event;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\EventDispatcher\ContainerAwareEventDispatcher;

/**
* Event listener
*/
class listener implements EventSubscriberInterface
{	
	/** @var \phpbb\auth\auth */
	protected $auth;
	
	/** @var \phpbb\config\config */
	protected $config;
	
	/** @var \phpbb\config\db_text */
	protected $config_text;
	
	/** @var \phpbb\request\request_interface */
	protected $request;

	/** @var \phpbb\template\template */
	protected $template;
	
	/** @var \phpbb\user */
	protected $user;
	
	/** @var \phpbb\db\driver\driver_interface */
	protected $db;
	
	/** @var string */
	protected $phpbb_root_path;
	protected $php_ext;

	/**
	* Constructor
	* 
	* @param \phpbb\auth\auth $auth
	* @param \phpbb\config\config $config
	* @param \phpbb\config\db_text $config_text
	* @param \phpbb\template\template $template
	* @param \phpbb\user $user
	* @param \phpbb\db\driver\driver_interface $db
	* @param string $phpbb_root_path Root path
	* @param string $php_ext
	*/
	
	public function __construct(\phpbb\auth\auth $auth, \phpbb\config\config $config, \phpbb\config\db_text $config_text, \phpbb\request\request_interface $request, \phpbb\template\template $template, \phpbb\user $user, \phpbb\db\driver\driver_interface $db, $phpbb_root_path, $php_ext)
	{
		$this->auth = $auth;
		$this->config = $config;
		$this->config_text = $config_text;
		$this->request = $request;
		$this->template = $template;
		$this->user = $user;
		$this->db = $db;
		$this->phpbb_root_path = $phpbb_root_path;
		$this->php_ext = $php_ext;
	}
	
	/**
	* Assign functions defined in this class to event listeners in the core
	*
	* @return array
	* @static
	* @access public
	*/
	static public function getSubscribedEvents()
	{
		return array(
			'core.user_setup'						=> 'hcode_user_setup',
			'core.page_header_after'				=> 'hcode_page_header_after',
			'core.modify_bbcode_init'				=> 'hcode_modify_bbcode_init',
			'core.validate_bbcode_by_extension'		=> 'hcode_validate_bbcode_by_extension',
			'core.bbcode_cache_init_end'			=> 'hcode_bbcode_cache_init_end',
			'core.bbcode_second_pass_by_extension'	=> 'hcode_bbcode_second_pass_by_extension',
		);
	}

	/**
	* Events
	*/
	private	$prosilver_code_open;
	private	$prosilver_code_close;
	private	$subsilver2_code_open;
	private	$subsilver2_code_close;
	private $bbcode;
	private $bbcode_id;

	public function hcode_user_setup($event)
	{
		$lang_set_ext = $event['lang_set_ext'];
		$lang_set_ext[] = array(
			'ext_name' => 'c61/hcode',
			'lang_set' => 'hcode',
		);
		$event['lang_set_ext'] = $lang_set_ext;

		$this->prosilver_code_open = htmlspecialchars_decode($this->config['hcode_prosilver_code_open']);
		$this->prosilver_code_close = htmlspecialchars_decode($this->config['hcode_prosilver_code_close']);
		$this->subsilver2_code_open = htmlspecialchars_decode($this->config['hcode_subsilver2_code_open']);
		$this->subsilver2_code_close = htmlspecialchars_decode($this->config['hcode_subsilver2_code_close']);
	}

	public function hcode_page_header_after($event)
	{
		$hcode_highlightjs_data = $this->config_text->get_array(array('hcode_highlightjs'));
		$hcode_prosilver_data = $this->config_text->get_array(array('hcode_prosilver'));
		$hcode_subsilver2_data = $this->config_text->get_array(array('hcode_subsilver2'));
		$hcode_hljs_script_data = $this->config_text->get_array(array('hcode_hljs_script'));

		$template_name = $this->user->style['style_path'];
		$super_template_name = rawurlencode((isset($this->user->style['style_parent_tree']) && $this->user->style['style_parent_tree']) ? $this->user->style['style_parent_tree'] : $this->user->style['style_path']);

		$this->template->assign_vars(array(
			'S_HCODE_PROSILVER'		=> ( ($super_template_name == 'prosilver') || ($template_name == 'prosilver') ) ? true : false,
			'S_HCODE_SUBSILVER2'	=> ( ($super_template_name == 'subsilver2') || ($template_name == 'subsilver2') ) ? true : false,

			'HCODE_HIGHLIGHTJS'		=> htmlspecialchars_decode($hcode_highlightjs_data['hcode_highlightjs']),
			'HCODE_AUTO'			=> (isset($this->config['hcode_auto']) && $this->config['hcode_auto']) ? $this->config['hcode_auto'] : '',
			
			'HCODE_EXPAND'			=> (isset($this->config['hcode_expand']) && $this->config['hcode_expand']) ? 1 : 0,
			'HCODE_LINENUMS'		=> (isset($this->config['hcode_linenums']) && $this->config['hcode_linenums']) ? 1 : 0,
			'HCODE_WRAP'			=> (isset($this->config['hcode_wrap']) && $this->config['hcode_wrap']) ? 1 : 0,

			'HCODE_PROSILVER_CODE_OPEN'		=> $this->prosilver_code_open,
			'HCODE_PROSILVER_CODE_CLOSE'	=> $this->prosilver_code_close,
			'HCODE_PROSILVER'		=> htmlspecialchars_decode($hcode_prosilver_data['hcode_prosilver']),

			'HCODE_SUBSILVER2_CODE_OPEN'	=> $this->subsilver2_code_open,
			'HCODE_SUBSILVER2_CODE_CLOSE'	=> $this->subsilver2_code_close,
			'HCODE_SUBSILVER2'		=> htmlspecialchars_decode($hcode_subsilver2_data['hcode_subsilver2']),

			'HCODE_HLJS_SCRIPT'		=> htmlspecialchars_decode($hcode_hljs_script_data['hcode_hljs_script']),
		));
	}

	public function hcode_modify_bbcode_init($event)
	{
		$bbcodes = $event['bbcodes'];
		$bbcodes['code'] = array(
			'bbcode_id' => 8,
			'regexp' => array(
				'#\[code(?:=([a-z0-9\-]+))?\](.+\[/code\])#uise' => "\$this->validate_bbcode_by_extension(\$this, '\$1', '\$2')"
			)
		);
		$event['bbcodes'] = $bbcodes;
	}

	public function hcode_validate_bbcode_by_extension($event)
	{
		$this->bbcode = $event['params_array'][0];
		$stx = $event['params_array'][1];
		$in = $event['params_array'][2];
		$in = str_replace("\r\n", "\n", str_replace('\"', '"', $in));
		if ( trim($in) == '' )
		{
			$out = $in;
		}
		else
		{
			// See phpbb 3.1.5, file includes/message_parser.php

			// We remove the hardcoded elements from the code block here because it is not used in code blocks
			// Having it here saves us one preg_replace per message containing [code] blocks
			// Additionally, magic url parsing should go after parsing bbcodes, but for safety those are stripped out too...
			$htm_match = get_preg_expression('bbcode_htm');
			unset($htm_match[4], $htm_match[5]);
			$htm_replace = array('\1', '\1', '\2', '\1');

			$out = $code_block = '';
			$open = 1;

			$str_from = array('<', '>', '[', ']', '.', ':');
			$str_to = array('&lt;', '&gt;', '&#91;', '&#93;', '&#46;', '&#58;');

			while ($in)
			{
				// Determine position and tag length of next code block
				preg_match('#(.*?)(\[code(?:=([a-z0-9\-]+))?\])(.+)#is', $in, $buffer);
				$pos = (isset($buffer[1])) ? strlen($buffer[1]) : false;
				$tag_length = (isset($buffer[2])) ? strlen($buffer[2]) : false;

				// Determine position of ending code tag
				$pos2 = stripos($in, '[/code]');

				// Which is the next block, ending code or code block
				if ($pos !== false && $pos < $pos2)
				{
					// Open new block
					if (!$open)
					{
						$out .= substr($in, 0, $pos);
						$in = substr($in, $pos);
						$stx = (isset($buffer[3])) ? $buffer[3] : '';
						$code_block = '';
					}
					else
					{
						// Already opened block, just append to the current block
						$code_block .= substr($in, 0, $pos) . ((isset($buffer[2])) ? $buffer[2] : '');
						$in = substr($in, $pos);
					}

					$in = substr($in, $tag_length);
					$open++;
				}
				else
				{
					// Close the block
					if ($open == 1)
					{
						$code_block .= substr($in, 0, $pos2);
						$code_block = preg_replace($htm_match, $htm_replace, $code_block);

						// Parse this code block
						$out .= '[code'. (empty($stx) ? '' : ('=' . $stx)) . ':' . $this->bbcode->bbcode_uid . ']' . str_replace($str_from, $str_to, $code_block) . '[/code:' . $this->bbcode->bbcode_uid . ']';
						$code_block = '';
						$open--;
					}
					else if ($open)
					{
						// Close one open tag... add to the current code block
						$code_block .= substr($in, 0, $pos2 + 7);
						$open--;
					}
					else
					{
						// end code without opening code... will be always outside code block
						$out .= substr($in, 0, $pos2 + 7);
					}

					$in = substr($in, $pos2 + 7);
				}
			}

			// if now $code_block has contents we need to parse the remaining code while removing the last closing tag to match up.
			if ($code_block)
			{
				$code_block = substr($code_block, 0, -7);
				$code_block = preg_replace($htm_match, $htm_replace, $code_block);

				$out .= '[code'. (empty($stx) ? '' : ('=' . $stx)) . ':' . $this->bbcode->bbcode_uid . ']' . str_replace($str_from, $str_to, $code_block) . '[/code:' . $this->bbcode->bbcode_uid . ']';
			}
		}
		$event['return'] = $out;
	}
	public function hcode_bbcode_cache_init_end($event)
	{
		$bbcache = $event['bbcode_cache'];
		$bbcache[8] = array(
			'preg' => array(
				'#\[code(?:=([a-z0-9\-]+))?:$uid\](.*?)\[/code:$uid\]#ise' => "\$this->bbcode_second_pass_by_extension('code_open', \$this, \$bbcode_id, '\$1', '\$2')"
			)
		);
		$event['bbcode_cache'] = $bbcache;
	}
	public function hcode_bbcode_second_pass_by_extension($event)
	{
		$mode = $event['params_array'][0];
		$this->bbcode = $event['params_array'][1];
		$this->bbcode_id = $event['params_array'][2];
		switch ($mode) {
			case 'code_open':
				$type = $event['params_array'][3];
				if ( $type == 'auto' )
				{
					$type = '';
				}
				$code = $event['params_array'][4];
				$code = str_replace('\"', '"', $code);
				$code = str_replace("\n", "\r", $code);
				while (!empty($code))
				{
					if (!($code[0] == "\r")) break;
					$code = substr($code, 1);
				}
				$code_open = $this->bbcode->bbcode_tpl('code_open');
				$code_close = $this->bbcode->bbcode_tpl('code_close');
				if ( !((strpos($code_open,'<pre') !== false) && (strpos($code_open,'<code') !== false)) )
				{
					if ( strpos($code_open,'<code') !== false )
					{
						// prosilver based
						$code_open = $this->prosilver_code_open;
						$code_close = $this->prosilver_code_close;
					}
					else if ( strpos($code_open,'<pre') !== false )
					{
						// subsilver2 based
						$code_open = $this->subsilver2_code_open;
						$code_close = $this->subsilver2_code_close;
					}
					else
					{
						$type = '';
					}
					if ( !empty($type) )
					{
						$code_open = str_replace(
								'<code>',
								'<code class="'.$type.'">',
								$code_open
						);

					}
				}
				$code = $code_open . $code . $code_close;
				$event['return'] = str_replace(
					array(
						'{L_CODE}',
						'{L_COLON}',
						'{L_SELECT_ALL_CODE}'
					),
					array(
						$this->user->lang['CODE'],
						$this->user->lang['COLON'],
						$this->user->lang['SELECT_ALL_CODE']
					),
					$code
				);
				break;
		}
	}
}
